# -*- coding: utf-8 -*-
import json, re
import ssl
import xml.etree.ElementTree as et
try:
    from urllib2 import urlopen, Request, build_opener, HTTPRedirectHandler
except ImportError:
    from urllib.request import urlopen, Request, build_opener, HTTPRedirectHandler
try:
    from urlparse import urlparse, urlunparse, parse_qs
except ImportError:
    from urllib.parse import urlparse, urlunparse, parse_qs
try:
    from urllib import urlencode
except ImportError:
    from urllib.parse import urlencode
ua = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.164 Safari/537.36'


def gethttp(url):
    try:
        ssl._create_default_https_context = ssl._create_unverified_context
    except:
        pass
    request = Request(url, headers={'User-Agent': ua})
    return urlopen(request).read().decode('utf-8')


def getjson(url):
    return json.loads(gethttp(url))


def getxml(url):
    content = gethttp(url)
    content = xml_fix(content)
    return et.fromstring(content.encode('utf-8'))


def xml_fix(content):
    ret = ''
    while True:
        offset = content.find('&')
        if offset == -1:
            ret += content
            return ret
        ret += content[:offset]
        content = content[offset:]
        offset = content.find(';')
        if offset < 0 or offset > 10:
            # print(content[:10])
            ret += '&amp;'
            content = content[1:]
        else:
            ret += '&'
            content = content[1:]


def remove_tag(content):
    content = re.sub(r'<javascript.*?</javascript.*?>', '', content)
    content = re.sub(r'<script.*?</script.*?>', '', content)
    content = re.sub(r'<.*?>', '', content)
    return content


def entry(url, s=False):
    try:
        content = gethttp(url)
    except Exception as e:
        return {'code': 'yszy 100', 'error': str(e)}
    t = data = root = None
    if content.startswith('{'):
        try:
            data = json.loads(gethttp(url))
        except Exception as e:
            return {'code': 'yszy 101', 'error': str(e)}
        t = 'json'
    if content.startswith('<'):
        content = xml_fix(content)
        try:
            root = et.fromstring(content.encode('utf-8'))
        except Exception as e:
            return {'code': 'yszy 102', 'error': str(e)}
        t = 'xml'
    o = urlparse(url)
    q = parse_qs(o.query, True)
    if t == 'json':
        if not q:
            return json_init(url, data, s)
        if 'ac' in q and q['ac'][0] == 'videolist':
            return json_videolist(url, data)
    if t == 'xml':
        if not q:
            return xml_init(url, root, s)
        if 'ac' in q and q['ac'][0] == 'videolist':
            return xml_videolist(url, root)
        if 'ac' in q and q['ac'][0] == 'list':
            return xml_list(url, root)
    return {'code': 'yszy 110'}


def json_init(url, data, s):
    ret = [{'All': url + '?ac=videolist'}]
    if s:
        ret.append({'Search': url + '?ac=videolist&wd='})
    for e in data['class']:
        ret.append({e['type_name']: url + '?ac=videolist&t=' + str(e['type_id'])})
    return ret


def json_videolist(url, data):
    ret = []
    for video in data['list']:
        plot = []
        plot.append(video['vod_name'])
        if video['vod_time']:
            plot.append(video['vod_time'])
        if video['vod_score'] and float(video['vod_score']):
            plot.append(video['vod_score'])
        if video['vod_remarks']:
            plot.append(u'状态：%s' % video['vod_remarks'])
        if video['type_name']:
            plot.append(u'类型：%s' % video['type_name'])
        if video['vod_area']:
            plot.append(u'地区：%s' % video['vod_area'])
        if video['vod_year']:
            plot.append(u'年份：%s' % video['vod_year'])
        if video['vod_lang']:
            plot.append(u'语言：%s' % video['vod_lang'])
        if video['vod_total']:
            plot.append(u'集数：%s' % video['vod_total'])
        if video['vod_duration']:
            plot.append(u'时长：%s' % video['vod_duration'])
        if video['vod_director']:
            plot.append(u'导演：%s' % video['vod_director'])
        if video['vod_actor']:
            plot.append(u'主演：%s' % video['vod_actor'])
        if video['vod_content']:
            plot.append(remove_tag(video['vod_content']))
        plot = '\n'.join(plot)
        ret2 = []
        arr = video['vod_play_from'].split('$$$')
        i = 0
        for dd in video['vod_play_url'].split('$$$'):
            nm = arr[i]
            i += 1
            ret3 = []
            for e in dd.split('#'):
                a = e.split('$')
                if not a:
                    continue
                if a[0].endswith('.m3u8'):
                    nm2 = 'Play'
                    url2 = a[0]
                elif len(a) > 1 and a[1].endswith('.m3u8'):
                    nm2 = a[0]
                    url2 = a[1]
                else:
                    continue
                ret3.append({nm2: url2})
            if not ret3:
                continue
            ret2.append({nm: ret3})
        if len(ret2) == 1:
            ret2 = ret3
        ret.append({'title': video['vod_name'], 'json': json.dumps(ret2), 'img': video['vod_pic'], 'plot': plot})
    o = urlparse(url)
    q = parse_qs(o.query, True)
    pg = int(data['page'])
    total_pg = int(data['pagecount'])
    if pg < total_pg:
        pg += 1
        q['pg'] = [pg]
        d = {}
        for k in q:
            d[k] = q[k][0]
        query = urlencode(d)
        url2 = urlunparse((o.scheme, o.netloc, o.path, o.params, query, o.fragment))
        ret.append({'Next Page (%d/%d)' % (pg, total_pg): url2})
    return ret


def xml_init(url, root, s):
    ret = [{'All': url + '?ac=videolist'}]
    if s:
        ret.append({'Search': url + '?ac=list&wd='})
    for e in root.findall('class/ty'):
        ret.append({e.text: url + '?ac=videolist&t=' + str(e.get('id'))})
    return ret


def xml_list(url, root):
    ids = []
    for video in root.findall('list/video'):
        ids.append(video.find('id').text)
    if not ids:
        return []
    ids = ','.join(ids)
    o = urlparse(url)
    q = parse_qs(o.query, True)
    d = {'ac': 'videolist', 'ids': ids}
    query = urlencode(d)
    url2 = urlunparse((o.scheme, o.netloc, o.path, o.params, query, o.fragment))
    ret = entry(url2)
    d = {}
    total_pg = int(root.find('list').get('pagecount'))
    pg = 1
    if 'pg' in q:
        pg = int(q['pg'][0])
    if pg < total_pg:
        pg += 1
        q['pg'] = [pg]
        for k in q:
            d[k] = q[k][0]
        query = urlencode(d)
        url2 = urlunparse((o.scheme, o.netloc, o.path, o.params, query, o.fragment))
        ret.append({'Next Page (%d/%d)' % (pg, total_pg): url2})
    return ret


def xml_videolist(url, root):
    ret = []
    for video in root.findall('list/video'):
        plot = []
        plot.append(video.find('name').text)
        if video.find('last').text:
            plot.append(video.find('last').text)
        if video.find('note').text:
            plot.append(u'状态：%s' % video.find('note').text)
        if video.find('type').text:
            plot.append(u'类型：%s' % video.find('type').text)
        if video.find('area').text:
            plot.append(u'地区：%s' % video.find('area').text)
        if video.find('year').text:
            plot.append(u'年份：%s' % video.find('year').text)
        if video.find('lang').text:
            plot.append(u'语言：%s' % video.find('lang').text)
        if video.find('director').text:
            plot.append(u'导演：%s' % video.find('director').text)
        if video.find('actor').text:
            plot.append(u'主演：%s' % video.find('actor').text)
        if video.find('des').text:
            plot.append(remove_tag(video.find('des').text))
        plot = '\n'.join(plot)
        ret2 = []
        for dd in video.findall('.//dl/dd'):
            nm = dd.get('flag')
            ret3 = []
            for e in dd.text.split('#'):
                a = e.split('$')
                if not a:
                    continue
                if a[0].endswith('.m3u8'):
                    nm2 = 'Play'
                    url2 = a[0]
                elif len(a) > 1 and a[1].endswith('.m3u8'):
                    nm2 = a[0]
                    url2 = a[1]
                else:
                    continue
                ret3.append({nm2: url2})
            if not ret3:
                continue
            ret2.append({nm: ret3})
        if len(ret2) == 1:
            ret2 = ret3
        ret.append({'title': video.find('name').text, 'json': json.dumps(ret2), 'img': video.find('pic').text, 'plot': plot})
    o = urlparse(url)
    q = parse_qs(o.query, True)
    pg = int(root.find('list').get('page'))
    total_pg = int(root.find('list').get('pagecount'))
    if pg < total_pg:
        pg += 1
        q['pg'] = [pg]
        d = {}
        for k in q:
            d[k] = q[k][0]
        query = urlencode(d)
        url2 = urlunparse((o.scheme, o.netloc, o.path, o.params, query, o.fragment))
        ret.append({'Next Page (%d/%d)' % (pg, total_pg): url2})
    return ret
